package schema

import (
	"entgo.io/ent"
	"entgo.io/ent/schema/edge"
	"entgo.io/ent/schema/field"
)

// MatchPlayer holds the schema definition for the MatchPlayer entity.
type MatchPlayer struct {
	ent.Schema
}

// Fields of the Stats.
func (MatchPlayer) Fields() []ent.Field {
	return []ent.Field{
		field.Int("team_id"),
		field.Int("kills"),
		field.Int("deaths"),
		field.Int("assists"),
		field.Int("headshot"),
		field.Uint("mvp"),
		field.Int("score"),
		field.Int("rank_new").Optional(),
		field.Int("rank_old").Optional(),
		field.Uint("mk_2").Optional(),
		field.Uint("mk_3").Optional(),
		field.Uint("mk_4").Optional(),
		field.Uint("mk_5").Optional(),
		field.Uint("dmg_enemy").Optional(),
		field.Uint("dmg_team").Optional(),
		field.Uint("ud_he").Optional(),
		field.Uint("ud_flames").Optional(),
		field.Uint("ud_flash").Optional(),
		field.Uint("ud_decoy").Optional(),
		field.Uint("ud_smoke").Optional(),
		field.String("crosshair").Optional(),
		field.Enum("color").Optional().Values("green", "yellow", "purple", "blue", "orange", "grey"),
		field.Int("kast").Optional(),
		field.Float32("flash_duration_self").Optional(),
		field.Float32("flash_duration_team").Optional(),
		field.Float32("flash_duration_enemy").Optional(),
		field.Uint("flash_total_self").Optional(),
		field.Uint("flash_total_team").Optional(),
		field.Uint("flash_total_enemy").Optional(),
		field.Uint64("match_stats").Optional(),
		field.Uint64("player_stats").Optional(),
		field.Int("flash_assists").Optional(),
		field.Float("avg_ping").Optional(),
	}
}

// Edges of the MatchPlayer.
func (MatchPlayer) Edges() []ent.Edge {
	return []ent.Edge{
		edge.From("matches", Match.Type).Ref("stats").Unique().Field("match_stats"),
		edge.From("players", Player.Type).Ref("stats").Unique().Field("player_stats"),
		edge.To("weapon_stats", Weapon.Type),
		edge.To("round_stats", RoundStats.Type),
		edge.To("spray", Spray.Type),
		edge.To("messages", Messages.Type),
	}
}
